@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -21,10 +21,17 @@
 extern "C" {
 #endif
 
+#include "lilv_config.h"
+
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+#include "sord/sord.h"
+#include "zix/tree.h"
+
+#include <float.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <float.h>
 
 #ifdef _WIN32
 #    include <windows.h>
@@ -52,16 +59,8 @@ static inline const char* dlerror(void) { return "Unknown error"; }
 #    include <unistd.h>
 #endif
 
-#include "serd/serd.h"
-#include "sord/sord.h"
-
-#include "zix/tree.h"
-
-#include "lilv_config.h"
-#include "lilv/lilv.h"
-
 #ifdef LILV_DYN_MANIFEST
-#    include "lv2/lv2plug.in/ns/ext/dynmanifest/dynmanifest.h"
+#    include "lv2/dynmanifest/dynmanifest.h"
 #endif
 
 /*
@@ -147,8 +146,9 @@ struct LilvInstancePimpl {
 };
 
 typedef struct {
-	bool dyn_manifest;
-	bool filter_language;
+	bool  dyn_manifest;
+	bool  filter_language;
+	char* lv2_path;
 } LilvOptions;
 
 struct LilvWorldImpl {
@@ -383,7 +383,7 @@ char*  lilv_get_lang(void);
 char*  lilv_expand(const char* path);
 char*  lilv_dirname(const char* path);
 int    lilv_copy_file(const char* src, const char* dst);
-bool   lilv_path_exists(const char* path, void* ignored);
+bool   lilv_path_exists(const char* path, const void* ignored);
 char*  lilv_path_absolute(const char* path);
 bool   lilv_path_is_absolute(const char* path);
 char*  lilv_get_latest_copy(const char* path, const char* copy_path);
@@ -398,7 +398,8 @@ bool   lilv_file_equals(const char* a_path, const char* b_path);
 
 char*
 lilv_find_free_path(const char* in_path,
-                    bool (*exists)(const char*, void*), void* user_data);
+                    bool (*exists)(const char*, const void*),
+                    const void* user_data);
 
 void
 lilv_dir_for_each(const char* path,
