@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -16,19 +16,30 @@
 
 #define __STDC_LIMIT_MACROS
 
-#include <assert.h>
+#include "lilv_config.h"
+#include "lilv_internal.h"
+
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+#include "sord/sord.h"
+#include "zix/tree.h"
+
+#include "lv2/core/lv2.h"
+#include "lv2/ui/ui.h"
+
+#ifdef LILV_DYN_MANIFEST
+#    include "lv2/dynmanifest/dynmanifest.h"
+#    include <dlfcn.h>
+#endif
+
 #include <math.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "lilv_config.h"
-#include "lilv_internal.h"
-
-#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
-
 #define NS_DOAP (const uint8_t*)"http://usefulinc.com/ns/doap#"
 #define NS_FOAF (const uint8_t*)"http://xmlns.com/foaf/0.1/"
 
@@ -720,10 +731,19 @@ lilv_plugin_get_port_by_designation(const LilvPlugin* plugin,
 LILV_API uint32_t
 lilv_plugin_get_latency_port_index(const LilvPlugin* plugin)
 {
+	LilvNode* lv2_OutputPort =
+		lilv_new_uri(plugin->world, LV2_CORE__OutputPort);
+	LilvNode* lv2_latency =
+		lilv_new_uri(plugin->world, LV2_CORE__latency);
+
 	const LilvPort* prop_port = lilv_plugin_get_port_by_property(
 		plugin, plugin->world->uris.lv2_reportsLatency);
-	const LilvPort* des_port = lilv_plugin_get_port_by_property(
-		plugin, plugin->world->uris.lv2_latency);
+	const LilvPort* des_port = lilv_plugin_get_port_by_designation(
+		plugin, lv2_OutputPort, lv2_latency);
+
+	lilv_node_free(lv2_latency);
+	lilv_node_free(lv2_OutputPort);
+
 	if (prop_port) {
 		return prop_port->index;
 	} else if (des_port) {
