@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2017 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,18 +14,28 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
-
-#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
-#include "lv2/lv2plug.in/ns/ext/atom/forge.h"
-#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-
-#include "lilv_config.h"
 #include "lilv_internal.h"
+
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+#include "sord/sord.h"
 #include "sratom/sratom.h"
+#include "zix/tree.h"
+
+#include "lv2/atom/atom.h"
+#include "lv2/atom/forge.h"
+#include "lv2/core/lv2.h"
+#include "lv2/presets/presets.h"
+#include "lv2/state/state.h"
+#include "lv2/urid/urid.h"
+
+#include <assert.h>
+#include <errno.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 #define USTR(s) ((const uint8_t*)(s))
 
@@ -38,10 +48,8 @@ typedef struct {
 } Property;
 
 typedef struct {
-	char*    symbol;  ///< Symbol of port
-	void*    value;   ///< Value of port
-	uint32_t size;    ///< Size of value
-	uint32_t type;    ///< Type of value (URID)
+	char*     symbol; ///< Symbol of port
+	LV2_Atom* atom;   ///< Value in port
 } PortValue;
 
 typedef struct {
@@ -58,7 +66,7 @@ struct LilvStateImpl {
 	LilvNode*     plugin_uri;   ///< Plugin URI
 	LilvNode*     uri;          ///< State/preset URI
 	char*         dir;          ///< Save directory (if saved)
-	char*         file_dir;     ///< Directory for files created by plugin
+	char*         scratch_dir;  ///< Directory for files created by plugin
 	char*         copy_dir;     ///< Directory for snapshots of external files
 	char*         link_dir;     ///< Directory for links to external files
 	char*         label;        ///< State/Preset label
@@ -115,12 +123,13 @@ append_port_value(LilvState*  state,
 	if (value) {
 		state->values = (PortValue*)realloc(
 			state->values, (++state->n_values) * sizeof(PortValue));
-		pv         = &state->values[state->n_values - 1];
-		pv->symbol = lilv_strdup(port_symbol);
-		pv->value  = malloc(size);
-		pv->size   = size;
-		pv->type   = type;
-		memcpy(pv->value, value, size);
+
+		pv             = &state->values[state->n_values - 1];
+		pv->symbol     = lilv_strdup(port_symbol);
+		pv->atom       = (LV2_Atom*)malloc(sizeof(LV2_Atom) + size);
+		pv->atom->size = size;
+		pv->atom->type = type;
+		memcpy(pv->atom + 1, value, size);
 	}
 	return pv;
 }
@@ -162,6 +171,18 @@ append_property(LilvState*     state,
 	prop->flags = flags;
 }
 
+static const Property*
+find_property(const LilvState* const state, const uint32_t key)
+{
+	const Property search_key = {NULL, 0, key, 0, 0};
+
+	return (const Property*)bsearch(&search_key,
+	                                state->props.props,
+	                                state->props.n,
+	                                sizeof(Property),
+	                                property_cmp);
+}
+
 static LV2_State_Status
 store_callback(LV2_State_Handle handle,
                uint32_t         key,
@@ -171,6 +192,15 @@ store_callback(LV2_State_Handle handle,
                uint32_t         flags)
 {
 	LilvState* const state = (LilvState*)handle;
+
+	if (!key) {
+		return LV2_STATE_ERR_UNKNOWN; // TODO: Add status for bad arguments
+	}
+
+	if (find_property((const LilvState*)handle, key)) {
+		return LV2_STATE_ERR_UNKNOWN; // TODO: Add status for duplicate keys
+	}
+
 	append_property(state, &state->props, key, value, size, type, flags);
 	return LV2_STATE_SUCCESS;
 }
@@ -182,11 +212,7 @@ retrieve_callback(LV2_State_Handle handle,
                   uint32_t*        type,
                   uint32_t*        flags)
 {
-	const LilvState* const state      = (LilvState*)handle;
-	const Property         search_key = { NULL, 0, key, 0, 0 };
-	const Property* const  prop       = (Property*)bsearch(
-		&search_key, state->props.props, state->props.n,
-		sizeof(Property), property_cmp);
+	const Property* const prop = find_property((const LilvState*)handle, key);
 
 	if (prop) {
 		*size  = prop->size;
@@ -198,9 +224,9 @@ retrieve_callback(LV2_State_Handle handle,
 }
 
 static bool
-lilv_state_has_path(const char* path, void* state)
+lilv_state_has_path(const char* path, const void* state)
 {
-	return lilv_state_rel2abs((LilvState*)state, path) != path;
+	return lilv_state_rel2abs((const LilvState*)state, path) != path;
 }
 
 static char*
@@ -232,9 +258,9 @@ abstract_path(LV2_State_Map_Path_Handle handle,
 	} else if (lilv_path_is_child(real_path, state->dir)) {
 		// File in state directory (loaded, or created by plugin during save)
 		path = lilv_path_relative_to(real_path, state->dir);
-	} else if (lilv_path_is_child(real_path, state->file_dir)) {
+	} else if (lilv_path_is_child(real_path, state->scratch_dir)) {
 		// File created by plugin earlier
-		path = lilv_path_relative_to(real_path, state->file_dir);
+		path = lilv_path_relative_to(real_path, state->scratch_dir);
 		if (state->copy_dir) {
 			int st = lilv_mkdir_p(state->copy_dir);
 			if (st) {
@@ -347,7 +373,7 @@ LILV_API LilvState*
 lilv_state_new_from_instance(const LilvPlugin*          plugin,
                              LilvInstance*              instance,
                              LV2_URID_Map*              map,
-                             const char*                file_dir,
+                             const char*                scratch_dir,
                              const char*                copy_dir,
                              const char*                link_dir,
                              const char*                save_dir,
@@ -359,14 +385,14 @@ lilv_state_new_from_instance(const LilvPlugin*          plugin,
 	const LV2_Feature** sfeatures = NULL;
 	LilvWorld* const    world     = plugin->world;
 	LilvState* const    state     = (LilvState*)calloc(1, sizeof(LilvState));
-	state->plugin_uri = lilv_node_duplicate(lilv_plugin_get_uri(plugin));
-	state->abs2rel    = zix_tree_new(false, abs_cmp, NULL, path_rel_free);
-	state->rel2abs    = zix_tree_new(false, rel_cmp, NULL, NULL);
-	state->file_dir   = file_dir ? absolute_dir(file_dir) : NULL;
-	state->copy_dir   = copy_dir ? absolute_dir(copy_dir) : NULL;
-	state->link_dir   = link_dir ? absolute_dir(link_dir) : NULL;
-	state->dir        = save_dir ? absolute_dir(save_dir) : NULL;
-	state->atom_Path  = map->map(map->handle, LV2_ATOM__Path);
+	state->plugin_uri  = lilv_node_duplicate(lilv_plugin_get_uri(plugin));
+	state->abs2rel     = zix_tree_new(false, abs_cmp, NULL, path_rel_free);
+	state->rel2abs     = zix_tree_new(false, rel_cmp, NULL, NULL);
+	state->scratch_dir = scratch_dir ? absolute_dir(scratch_dir) : NULL;
+	state->copy_dir    = copy_dir ? absolute_dir(copy_dir) : NULL;
+	state->link_dir    = link_dir ? absolute_dir(link_dir) : NULL;
+	state->dir         = save_dir ? absolute_dir(save_dir) : NULL;
+	state->atom_Path   = map->map(map->handle, LV2_ATOM__Path);
 
 	LV2_State_Map_Path  pmap          = { state, abstract_path, absolute_path };
 	LV2_Feature         pmap_feature  = { LV2_STATE__mapPath, &pmap };
@@ -424,8 +450,9 @@ lilv_state_emit_port_values(const LilvState*     state,
                             void*                user_data)
 {
 	for (uint32_t i = 0; i < state->n_values; ++i) {
-		const PortValue* val = &state->values[i];
-		set_value(val->symbol, user_data, val->value, val->size, val->type);
+		const PortValue* value = &state->values[i];
+		const LV2_Atom*  atom  = value->atom;
+		set_value(value->symbol, user_data, atom + 1, atom->size, atom->type);
 	}
 }
 
@@ -470,6 +497,16 @@ lilv_state_restore(const LilvState*           state,
 	}
 }
 
+static void
+set_state_dir_from_model(LilvState* state, const SordNode* graph)
+{
+	if (!state->dir && graph) {
+		const char* uri = (const char*)sord_node_get_string(graph);
+		state->dir = lilv_file_uri_parse(uri, NULL);
+	}
+	assert(!state->dir || lilv_path_is_absolute(state->dir));
+}
+
 static LilvState*
 new_state_from_model(LilvWorld*       world,
                      LV2_URID_Map*    map,
@@ -494,9 +531,7 @@ new_state_from_model(LilvWorld*       world,
 		const SordNode* object = sord_iter_get_node(i, SORD_OBJECT);
 		const SordNode* graph  = sord_iter_get_node(i, SORD_GRAPH);
 		state->plugin_uri = lilv_node_new_from_node(world, object);
-		if (!state->dir && graph) {
-			state->dir = lilv_strdup((const char*)sord_node_get_string(graph));
-		}
+		set_state_dir_from_model(state, graph);
 		sord_iter_free(i);
 	} else if (sord_ask(model,
 	                    node,
@@ -515,9 +550,7 @@ new_state_from_model(LilvWorld*       world,
 		const SordNode* object = sord_iter_get_node(i, SORD_OBJECT);
 		const SordNode* graph  = sord_iter_get_node(i, SORD_GRAPH);
 		state->label = lilv_strdup((const char*)sord_node_get_string(object));
-		if (!state->dir && graph) {
-			state->dir = lilv_strdup((const char*)sord_node_get_string(graph));
-		}
+		set_state_dir_from_model(state, graph);
 		sord_iter_free(i);
 	}
 
@@ -602,7 +635,7 @@ new_state_from_model(LilvWorld*       world,
 	sord_node_free(world->world, state_node);
 	sord_node_free(world->world, statep);
 
-	free((void*)chunk.buf);
+	serd_free((void*)chunk.buf);
 	sratom_free(sratom);
 
 	if (state->props.props) {
@@ -872,7 +905,7 @@ add_state_to_manifest(LilvWorld*      lworld,
 }
 
 static bool
-link_exists(const char* path, void* data)
+link_exists(const char* path, const void* data)
 {
 	const char* target = (const char*)data;
 	if (!lilv_path_exists(path, NULL)) {
@@ -884,6 +917,12 @@ link_exists(const char* path, void* data)
 	return !matches;
 }
 
+static int
+maybe_symlink(const char* oldpath, const char* newpath)
+{
+  return link_exists(newpath, oldpath) ? 0 : lilv_symlink(oldpath, newpath);
+}
+
 static void
 write_property_array(const LilvState*     state,
                      const PropertyArray* array,
@@ -988,7 +1027,7 @@ lilv_state_write(LilvWorld*       world,
 		// _:symbol pset:value value
 		p = serd_node_from_string(SERD_URI, USTR(LV2_PRESETS__value));
 		sratom_write(sratom, unmap, SERD_ANON_CONT, &port, &p,
-		             value->type, value->size, value->value);
+		             value->atom->type, value->atom->size, value->atom + 1);
 
 		serd_writer_end_anon(writer, &port);
 	}
@@ -1026,7 +1065,7 @@ lilv_state_make_links(const LilvState* state, const char* dir)
 		    && strcmp(state->copy_dir, dir)) {
 			// Link directly to snapshot in the copy directory
 			char* target = lilv_path_relative_to(pm->abs, dir);
-			lilv_symlink(target, path);
+			maybe_symlink(target, path);
 			free(target);
 		} else if (!lilv_path_is_child(pm->abs, dir)) {
 			const char* link_dir = state->link_dir ? state->link_dir : dir;
@@ -1034,7 +1073,7 @@ lilv_state_make_links(const LilvState* state, const char* dir)
 			if (!strcmp(dir, link_dir)) {
 				// Link directory is save directory, make link at exact path
 				remove(pat);
-				lilv_symlink(pm->abs, pat);
+				maybe_symlink(pm->abs, pat);
 			} else {
 				// Make a link in the link directory to external file
 				char* lpath = lilv_find_free_path(pat, link_exists, pm->abs);
@@ -1044,7 +1083,7 @@ lilv_state_make_links(const LilvState* state, const char* dir)
 
 				// Make a link in the save directory to the external link
 				char* target = lilv_path_relative_to(lpath, dir);
-				lilv_symlink(target, path);
+				maybe_symlink(target, path);
 				free(target);
 				free(lpath);
 			}
@@ -1089,10 +1128,9 @@ lilv_state_save(LilvWorld*       world,
 		world, map, unmap, state, ttl, (const char*)node.buf, dir);
 
 	// Set saved dir and uri (FIXME: const violation)
-	SerdNode dir_uri = serd_node_new_file_uri(USTR(abs_dir), NULL, NULL, true);
 	free(state->dir);
 	lilv_node_free(state->uri);
-	((LilvState*)state)->dir = (char*)dir_uri.buf;
+	((LilvState*)state)->dir = lilv_strdup(abs_dir);
 	((LilvState*)state)->uri = lilv_new_uri(world, (const char*)node.buf);
 
 	serd_node_free(&file);
@@ -1132,7 +1170,18 @@ lilv_state_to_string(LilvWorld*       world,
 
 	serd_writer_free(writer);
 	serd_env_free(env);
-	return (char*)serd_chunk_sink_finish(&chunk);
+	char* str    = (char*)serd_chunk_sink_finish(&chunk);
+	char* result = lilv_strdup(str);
+	serd_free(str);
+	return result;
+}
+
+static void
+try_unlink(const char* path)
+{
+	if (unlink(path)) {
+		LILV_ERRORF("Failed to remove %s (%s)\n", path, strerror(errno));
+	}
 }
 
 LILV_API int
@@ -1144,7 +1193,7 @@ lilv_state_delete(LilvWorld*       world,
 		return -1;
 	}
 
-	LilvNode*  bundle        = lilv_new_uri(world, state->dir);
+	LilvNode*  bundle        = lilv_new_file_uri(world, NULL, state->dir);
 	LilvNode*  manifest      = lilv_world_get_manifest_uri(world, bundle);
 	char*      manifest_path = lilv_node_get_path(manifest, NULL);
 	SordModel* model         = sord_new(world->world, SORD_SPO, false);
@@ -1162,12 +1211,10 @@ lilv_state_delete(LilvWorld*       world,
 		model, state->uri->node, world->uris.rdfs_seeAlso, NULL, NULL);
 	if (file) {
 		// Remove state file
-		char* path = lilv_file_uri_parse(
-			(const char*)sord_node_get_string(file), NULL);
-		if (unlink(path)) {
-			LILV_ERRORF("Failed to remove %s (%s)\n", path, strerror(errno));
-		}
-		lilv_free(path);
+		char* path =
+			(char*)serd_file_uri_parse(sord_node_get_string(file), NULL);
+		try_unlink(path);
+		serd_free(path);
 	}
 
 	// Remove any existing manifest entries for this state
@@ -1181,16 +1228,22 @@ lilv_state_delete(LilvWorld*       world,
 
 	if (sord_num_quads(model) == 0) {
 		// Manifest is empty, attempt to remove bundle entirely
-		if (unlink(manifest_path)) {
-			LILV_ERRORF("Failed to remove %s (%s)\n",
-			            manifest_path, strerror(errno));
+		try_unlink(manifest_path);
+
+		// Remove all known files from state bundle
+		for (ZixTreeIter* i = zix_tree_begin(state->abs2rel);
+		     i != zix_tree_end(state->abs2rel);
+		     i = zix_tree_iter_next(i)) {
+			const PathMap* pm   = (const PathMap*)zix_tree_get(i);
+			char*          path = lilv_path_join(state->dir, pm->rel);
+			try_unlink(path);
+			free(path);
 		}
-		char* dir_path = lilv_file_uri_parse(state->dir, NULL);
-		if (rmdir(dir_path)) {
-			LILV_ERRORF("Failed to remove %s (%s)\n",
-			            dir_path, strerror(errno));
+
+		if (rmdir(state->dir)) {
+			LILV_ERRORF("Failed to remove directory %s (%s)\n",
+			            state->dir, strerror(errno));
 		}
-		lilv_free(dir_path);
 	} else {
 		// Still something in the manifest, reload bundle
 		lilv_world_load_bundle(world, bundle);
@@ -1224,7 +1277,7 @@ lilv_state_free(LilvState* state)
 		free_property_array(state, &state->props);
 		free_property_array(state, &state->metadata);
 		for (uint32_t i = 0; i < state->n_values; ++i) {
-			free(state->values[i].value);
+			free(state->values[i].atom);
 			free(state->values[i].symbol);
 		}
 		lilv_node_free(state->plugin_uri);
@@ -1234,7 +1287,7 @@ lilv_state_free(LilvState* state)
 		free(state->values);
 		free(state->label);
 		free(state->dir);
-		free(state->file_dir);
+		free(state->scratch_dir);
 		free(state->copy_dir);
 		free(state->link_dir);
 		free(state);
@@ -1256,9 +1309,10 @@ lilv_state_equals(const LilvState* a, const LilvState* b)
 	for (uint32_t i = 0; i < a->n_values; ++i) {
 		PortValue* const av = &a->values[i];
 		PortValue* const bv = &b->values[i];
-		if (av->size != bv->size || av->type != bv->type
-		    || strcmp(av->symbol, bv->symbol)
-		    || memcmp(av->value, bv->value, av->size)) {
+		if (av->atom->size != bv->atom->size ||
+		    av->atom->type != bv->atom->type ||
+		    strcmp(av->symbol, bv->symbol) ||
+		    memcmp(av->atom + 1, bv->atom + 1, av->atom->size)) {
 			return false;
 		}
 	}
