@@ -1,6 +1,13 @@
-#include "lilv/lilv.h"
 #include "../src/lilv_internal.h"
 
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #define PLUGIN_URI "http://example.org/failed-instantiation"
 
 #define TEST_ASSERT(check) do {\
